-- rgbled_arty.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity rgbled_arty is
  port
  (
    clk100mhz : in std_logic;
    btn : in std_logic_vector(1 downto 0);
    sw : in std_logic_vector(1 downto 0);
    led0_r : out std_logic;
    led0_g : out std_logic;
    led0_b : out std_logic;
    seg : out std_logic_vector(6 downto 0);
    cat : out std_logic
  );
    
end rgbled_arty;

architecture impl of rgbled_arty is
component pwm is
  port
  (
    clk : in std_logic;
    dc : in std_logic_vector(5 downto 0); -- duty cycle
    pwm : out std_logic
  );
end component;

component sseg_arty_2dig is
  port
  (
    clk : in std_logic;
    binval: in std_logic_vector(5 downto 0);
    seg : out std_logic_vector(6 downto 0);
    cat : out std_logic
  );
end component;


type dc_state_type is (idle, inc, dec);
signal dc_state_reg : dc_state_type := idle;
signal dc_state_next : dc_state_type;

type ta_state_type is (released, pressed, pulse);
signal ta_plus_state_reg : ta_state_type := released;
signal ta_plus_state_next : ta_state_type;

signal ta_minus_state_reg : ta_state_type := released;
signal ta_minus_state_next : ta_state_type;


signal ta_plus_pulse : std_logic := '0';
signal ta_minus_pulse : std_logic := '0';

signal redval_reg : unsigned(5 downto 0) := (others => '0');
signal redval_next : unsigned(5 downto 0);
signal greenval_reg : unsigned(5 downto 0) := (others => '0');
signal greenval_next : unsigned(5 downto 0);
signal blueval_reg : unsigned(5 downto 0) := (others => '0');
signal blueval_next : unsigned(5 downto 0);

signal ta_plus : std_logic;
signal ta_minus : std_logic;

signal colorval_disp : std_logic_vector(5 downto 0);


begin
  pwm_r : pwm
port map
(
  clk => clk100mhz,
  dc => std_logic_vector(redval_reg),
  pwm => led0_r
);

pwm_g : pwm
port map
(
  clk => clk100mhz,
  dc => std_logic_vector(greenval_reg),
  pwm => led0_g
);

pwm_b : pwm
port map
(
  clk => clk100mhz,
  dc => std_logic_vector(blueval_reg),
  pwm => led0_b
);
  
  
  disp : sseg_arty_2dig
  port map
  (
    clk => clk100mhz,
    binval => colorval_disp,
    seg => seg,
    cat => cat
  );
  

  upd_proc: process (clk100mhz)
  begin
    if rising_edge(clk100mhz) then
      dc_state_reg <= dc_state_next;
      ta_plus_state_reg <= ta_plus_state_next;
      ta_minus_state_reg <= ta_minus_state_next;
      redval_reg <= redval_next;
      greenval_reg <= greenval_next;
      blueval_reg <= blueval_next;
    end if;
  end process upd_proc;
  
  state_proc: process (dc_state_reg, redval_reg, greenval_reg, blueval_reg, ta_plus_pulse, ta_minus_pulse, sw)
  begin
    dc_state_next <= dc_state_reg; -- default, no change
    redval_next <= redval_reg; --default, no change
    greenval_next <= greenval_reg; --default, no change
    blueval_next <= blueval_reg; --default, no change

    case dc_state_reg is
      when idle =>
        if ta_plus_pulse = '1' then
          dc_state_next <= inc;
        elsif ta_minus_pulse = '1' then
          dc_state_next <= dec;
        end if;
      when inc =>
        if sw = "10" then
          redval_next <= redval_reg + 1;
        elsif sw = "01" then
          greenval_next <= greenval_reg + 1;
        elsif sw = "00" then
            blueval_next <= blueval_reg + 1;
        end if;  
        dc_state_next <= idle;
      when dec =>
        if sw = "10" then
        redval_next <= redval_reg - 1;
      elsif sw = "01" then
        greenval_next <= greenval_reg - 1;
      elsif sw = "00" then
          blueval_next <= blueval_reg - 1;
      end if;  
          dc_state_next <= idle;
    end case;
  end process state_proc;

  ta_plus_proc : process (ta_plus_state_reg, ta_plus)
  begin
    ta_plus_state_next <= ta_plus_state_reg; -- default
    ta_plus_pulse <= '0'; --default
    case ta_plus_state_reg is
      when released =>
        if ta_plus = '1' then
          ta_plus_state_next <= pressed;
        end if;
      when pressed =>
        if ta_plus = '0' then
          ta_plus_state_next <= pulse;
        end if;
      when pulse =>
        ta_plus_pulse <= '1';
        ta_plus_state_next <= released;
    end case;
  end process;

  ta_minus_proc : process (ta_minus_state_reg, ta_minus)
  begin
    ta_minus_state_next <= ta_minus_state_reg; -- default
    ta_minus_pulse <= '0'; --default
    case ta_minus_state_reg is
      when released =>
        if ta_minus = '1' then
          ta_minus_state_next <= pressed;
        end if;
      when pressed =>
        if ta_minus = '0' then
          ta_minus_state_next <= pulse;
        end if;
      when pulse =>
        ta_minus_pulse <= '1';
        ta_minus_state_next <= released;
    end case;
  end process;


  ta_plus <= btn(0);
  ta_minus <= btn(1); 
  
  with sw select
    colorval_disp <=
      std_logic_vector(blueval_reg) when "00",
      std_logic_vector(greenval_reg) when "01",
      std_logic_vector(redval_reg) when "10",
      "000000" when others;
end impl;
