-- digitalclock_basys3.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity digitalclock_basys3 is
 port
 (
  clk100 : in std_logic; -- 100MHz clock
  btnC : in std_logic; -- reset button
  btnU : in std_logic; -- hour set button
  btnR : in std_logic; -- minute set button
  led0 : out std_logic; -- heartbeat led
  seg : out std_logic_vector(6 downto 0); -- seven segement display
  dp : out std_logic; -- decimal point
  an : out std_logic_vector(3 downto 0) -- anode drivers (inverted)
  );
end digitalclock_basys3;

architecture impl of digitalclock_basys3 is


component debounce is
  generic
  (
    num_cons_cycles : integer := 2000;
    cntr_size : integer := 11
  );
  port
  (
    clk : in std_logic;
    data_in : in std_logic;
    data_deb : out std_logic
  );
end component;


component clkgen is
  port
  (
    clk_sys : in std_logic; -- system clock
    reset : in std_logic; -- reset
    hour_set : in std_logic; -- '1' = hour set mode
    minute_set : in std_logic; -- '1' = minute set mode
    sec_pulse : in std_logic; -- second pulse
    minutes : out std_logic_vector(5 downto 0); -- minutes output
    hours : out std_logic_vector(4 downto 0) -- hours out
  );
end component;

component calc_segdata is
  port
  (
    hours : in std_logic_vector(4 downto 0);
    minutes : in std_logic_vector(5 downto 0);
    ht_seg : out std_logic_vector(7 downto 0);
    ho_seg : out std_logic_vector(7 downto 0);
    mt_seg : out std_logic_vector(7 downto 0);
    mo_seg : out std_logic_vector(7 downto 0)   
  );    
end component;

component sseg_control is
  generic
  (
    cntr_size : integer := 20
  );
  port
  (
    clk : in std_logic;
    disp_data : in std_logic_vector(31 downto 0);
    seg_data : out std_logic_vector(7 downto 0);
    digit_sel : out std_logic_vector(3 downto 0)
  );
end component;

component trans_segdata_a is
  generic
  (
    cntr_size : integer := 20
  );
  port
  (
    clk : in std_logic;
    ht_seg : in std_logic_vector(7 downto 0);
    ho_seg : in std_logic_vector(7 downto 0);
    mt_seg : in std_logic_vector(7 downto 0);
    mo_seg : in std_logic_vector(7 downto 0);
    seg : out std_logic_vector(6 downto 0); -- seven segement display
    dp : out std_logic; -- decimal point
    an : out std_logic_vector(3 downto 0) -- anode drivers (inverted)
  );   
end component;

constant clk_cntr_max : integer := 100_000_000;
-- ceil(log2(cntr_max) = 27
constant clk_cntr_size : integer := 27;

signal clk : std_logic;

signal sec_pulse : std_logic;
signal clk_cntr_reg : unsigned(clk_cntr_size-1 downto 0) := (others => '0');
signal clk_cntr_next : unsigned(clk_cntr_size-1 downto 0);

signal reset : std_logic;
signal hour_set : std_logic;
signal minute_set : std_logic;
signal hours_buf : std_logic_vector(4 downto 0);
signal minutes_buf : std_logic_vector(5 downto 0);
 
signal ht_seg_data : std_logic_vector(7 downto 0);
signal ho_seg_data : std_logic_vector(7 downto 0);
signal mt_seg_data : std_logic_vector(7 downto 0);
signal mo_seg_data : std_logic_vector(7 downto 0);

begin

  clk <= clk100;
  
  process (clk)
  begin
    if rising_edge(clk) then
      clk_cntr_reg <= clk_cntr_next;
    end if;
  end process;
  clk_cntr_next <= (others => '0') when (clk_cntr_reg = (clk_cntr_max-1))
                                   else (clk_cntr_reg + 1);
  sec_pulse <= '1' when clk_cntr_reg = (clk_cntr_max-1) else '0';
  
  led0 <= clk_cntr_reg(clk_cntr_size-1);

  deb_rst : debounce
  generic map
  (
    num_cons_cycles => 2000,
    cntr_size => 11
  )
  port map
  (
    clk => clk,
    data_in => btnC,
    data_deb => reset
  );  

  deb_hourset : debounce
  generic map
  (
    num_cons_cycles => 2000,
    cntr_size => 11
  )
  port map
  (
    clk => clk,
    data_in => btnU,
    data_deb => hour_set
  );  

  deb_minuteset : debounce
  generic map
  (
    num_cons_cycles => 2000,
    cntr_size => 11
  )
  port map
  (
    clk => clk,
    data_in => btnR,
    data_deb => minute_set
  );  

  clkg : clkgen
  port map
  (
    clk_sys => clk,
    reset => reset,
    hour_set => hour_set,
    minute_set => minute_set,
    sec_pulse => sec_pulse,
    minutes => minutes_buf,
    hours => hours_buf
  );
  
  csegdata : calc_segdata
  port map
  (
    hours => hours_buf,
    minutes => minutes_buf,
    ht_seg => ht_seg_data,
    ho_seg => ho_seg_data,
    mt_seg => mt_seg_data,
    mo_seg => mo_seg_data
  );

  tr_data : trans_segdata_a
  port map
  (
    clk => clk,
    ht_seg => ht_seg_data,
    ho_seg => ho_seg_data,
    mt_seg => mt_seg_data,
    mo_seg => mo_seg_data,
    seg => seg,
    dp => dp,
    an => an
  );  
  
end impl;