-- dg_papilio.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dg_papilio is
  port
(
  clk32 : in std_logic; -- 100MHz clock
  joy_select : in std_logic; -- reset button, active low
  switch : in std_logic_vector(1 downto 0);
  -- 0: hour set 
  -- 1: minute set 
  led0 : out std_logic; -- heartbeat led
  seg7_a : out std_logic; -- seven segement display
  seg7_b : out std_logic; -- seven segement display
  seg7_c : out std_logic; -- seven segement display
  seg7_d : out std_logic; -- seven segement display
  seg7_e : out std_logic; -- seven segement display
  seg7_f : out std_logic; -- seven segement display
  seg7_g : out std_logic; -- seven segement display
  seg7_dp : out std_logic; -- decimal point
  seg7_an : out std_logic_vector(3 downto 0) -- anode drivers (inverted)
);
end dg_papilio;

architecture impl of dg_papilio is
component clk_mgn
port
 (-- Clock in ports
  clk_in1           : in     std_logic;
  -- Clock out ports
  clk_out1          : out    std_logic
 );
end component;

component debounce is
  generic
  (
    num_cons_cycles : integer := 2000;
    cntr_size : integer := 11
  );
  port
  (
    clk : in std_logic;
    data_in : in std_logic;
    data_deb : out std_logic
  );
end component;


component clkgen is
  port
  (
    clk_sys : in std_logic; -- system clock
    reset : in std_logic; -- reset
    hour_set : in std_logic; -- '1' = hour set mode
    minute_set : in std_logic; -- '1' = minute set mode
    sec_pulse : in std_logic; -- second pulse
    minutes : out std_logic_vector(5 downto 0); -- minutes output
    hours : out std_logic_vector(4 downto 0) -- hours out
  );
end component;

component calc_segdata is
  port
  (
    hours : in std_logic_vector(4 downto 0);
    minutes : in std_logic_vector(5 downto 0);
    ht_seg : out std_logic_vector(7 downto 0);
    ho_seg : out std_logic_vector(7 downto 0);
    mt_seg : out std_logic_vector(7 downto 0);
    mo_seg : out std_logic_vector(7 downto 0)   
  );    
end component;

component sseg_control is
  generic
  (
    cntr_size : integer := 20
  );
  port
  (
    clk : in std_logic;
    disp_data : in std_logic_vector(31 downto 0);
    seg_data : out std_logic_vector(7 downto 0);
    digit_sel : out std_logic_vector(3 downto 0)
  );
end component;

constant cntr_max : integer := 5_000_000;
-- ceil(log2(cntr_max) = 23
constant cntr_size : integer := 23;

signal clk5 : std_logic;
signal sec_pulse : std_logic;
signal cntr_reg : unsigned(cntr_size-1 downto 0) := (others => '0');
signal cntr_next : unsigned(cntr_size-1 downto 0);

signal reset : std_logic;
signal hour_set : std_logic;
signal minute_set : std_logic;
signal hours_buf : std_logic_vector(4 downto 0);
signal minutes_buf : std_logic_vector(5 downto 0);
 
signal dispdata_buf : std_logic_vector(31 downto 0);
signal dispdata_buf_n : std_logic_vector(31 downto 0);

signal seg_reg : std_logic_vector(7 downto 0);
signal seg_next : std_logic_vector(7 downto 0);
signal digitsel_reg : std_logic_vector(3 downto 0);
signal digitsel_next : std_logic_vector(3 downto 0);

signal reset_in : std_logic; 
 
begin

  reset_in <= not joy_select;

  clk_div : clk_mgn
  port map
  ( 
    clk_in1 => clk32,
    clk_out1 => clk5
  );

  process (clk5)
  begin
    if rising_edge(clk5) then
      cntr_reg <= cntr_next;
      seg_reg <= seg_next;
      digitsel_reg <= digitsel_next;
    end if;
  end process;
  cntr_next <= (others => '0') when (cntr_reg = (cntr_max-1)) else (cntr_reg + 1);
  sec_pulse <= '1' when cntr_reg = (cntr_max-1) else '0';
  led0 <= cntr_reg(cntr_size-1);


  deb_rst : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => reset_in,
    data_deb => reset
  );  

  deb_hourset : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => switch(0),
    data_deb => hour_set
  );  

  deb_minuteset : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => switch(1),
    data_deb => minute_set
  );  



  clkg : clkgen
  port map
  (
    clk_sys => clk5,
    reset => reset,
    hour_set => hour_set,
    minute_set => minute_set,
    sec_pulse => sec_pulse,
    minutes => minutes_buf,
    hours => hours_buf
  );
  
  csegdata : calc_segdata
  port map
  (
    hours => hours_buf,
    minutes => minutes_buf,
    ht_seg => dispdata_buf(31 downto 24),
    ho_seg => dispdata_buf(23 downto 16),
    mt_seg => dispdata_buf(15 downto 8),
    mo_seg => dispdata_buf(7 downto 0)
  );
  
  dispdata_buf_n <= not dispdata_buf;
  
  disp_control : sseg_control
  generic map
  (
    cntr_size => 16
  )
  port map
  (
    clk => clk5,
    disp_data => dispdata_buf_n,
    seg_data => seg_next,
    digit_sel => digitsel_next
  );
  
  seg7_a <= seg_reg(0);
  seg7_b <= seg_reg(1);
  seg7_c <= seg_reg(2);
  seg7_d <= seg_reg(3);
  seg7_e <= seg_reg(4);
  seg7_f <= seg_reg(5);
  seg7_g <= seg_reg(6);
  seg7_dp <= seg_reg(7);
  
  seg7_an(0) <= not digitsel_reg(3);
  seg7_an(1) <= not digitsel_reg(2);
  seg7_an(2) <= not digitsel_reg(1);
  seg7_an(3) <= not digitsel_reg(0);
  
end impl;
