-- pong_main.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity pong_main is
  port
  (
    clk100 : in std_logic;
    btnC : in std_logic;
    btnU : in std_logic;
    btnD : in std_logic;
    r0 : out std_logic;
    r1 : out std_logic;
    g0 : out std_logic;
    g1 : out std_logic;
    b0 : out std_logic;
    b1 : out std_logic;
    a : out std_logic;
    b : out std_logic;
    c : out std_logic;
    d : out std_logic;
    clk_pix : out std_logic;
    stb : out std_logic;
    oe_n : out std_logic
  );
end pong_main;

architecture impl of pong_main is
  component clk_wiz_0
  port
  (
    clk_in1 : in std_logic;
    clk_out1 : out std_logic
  ); 
end component;

component rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
   );
    
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic;
    oe_len : in std_logic_vector(rsize-1 downto 0);
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end component;

component debounce is
  generic
  (
    num_cons_cycles : integer := 2000;
    cntr_size : integer := 11
  );
  port
  (
    clk : in std_logic;
    data_in : in std_logic;
    data_deb : out std_logic
  );
end component;

constant x_size : integer := 64;
constant y_size : integer := 32;
constant rsize : integer := 7;

signal p_x : unsigned (rsize-1 downto 0);
signal p_y : unsigned (rsize-1 downto 0);
signal p_y_l : unsigned (rsize-1 downto 0);

signal x_addr : std_logic_vector(rsize-1 downto 0);
signal y_addr : std_logic_vector(rsize-1 downto 0);
signal disp_yaddr : std_logic_vector(3 downto 0);
signal disp_yaddr_minus_one : std_logic_vector(3 downto 0);

-- wall
constant wall_xl : integer := 1;
constant wall_xr : integer := 2;

-- paddle
constant paddle_x : integer := 55;
constant paddle_height : integer := 6;
constant paddle_d : integer := 1; -- paddle delta when moving

signal paddle_yt : unsigned(rsize-1 downto 0);
signal paddle_yb : unsigned(rsize-1 downto 0);
signal paddle_yreg : unsigned(rsize-1 downto 0);
signal paddle_ynext : unsigned(rsize-1 downto 0);

-- ball
constant ball_size : integer := 2;
constant ball_vp : unsigned(rsize-1 downto 0) := to_unsigned(1,rsize);
constant ball_vn : unsigned(rsize-1 downto 0) := unsigned(to_signed(-1,rsize));

-- position management
signal ball_xl : unsigned(rsize-1 downto 0);
signal ball_xr : unsigned(rsize-1 downto 0);
signal ball_yt : unsigned(rsize-1 downto 0);
signal ball_yb : unsigned(rsize-1 downto 0);
signal ball_xreg : unsigned(rsize-1 downto 0) := (others => '0');
signal ball_xnext : unsigned(rsize-1 downto 0);
signal ball_yreg : unsigned(rsize-1 downto 0) := (others => '0');
signal ball_ynext : unsigned(rsize-1 downto 0);

--displacement management
signal ball_dxreg : unsigned(rsize-1 downto 0) := to_unsigned(1, rsize);
signal ball_dxnext : unsigned(rsize-1 downto 0);
signal ball_dyreg : unsigned(rsize-1 downto 0) := to_unsigned(1, rsize);
signal ball_dynext : unsigned(rsize-1 downto 0);

-- image management
signal wall_active_u : std_logic;
signal paddle_active_u : std_logic;
signal ball_active_u : std_logic;

signal wall_active_l : std_logic;
signal paddle_active_l : std_logic;
signal ball_active_l : std_logic;

constant wall_color : std_logic_vector(2 downto 0) := "110"; --yellow
constant paddle_color : std_logic_vector(2 downto 0) := "100"; --red
constant ball_color : std_logic_vector(2 downto 0) := "010"; --green

signal obj_reg_u : std_logic_vector(2 downto 0) := "000";
signal obj_next_u : std_logic_vector(2 downto 0);

signal obj_reg_l : std_logic_vector(2 downto 0) := "000";
signal obj_next_l : std_logic_vector(2 downto 0);

signal clk5 : std_logic;
signal reset : std_logic;
signal btn_up : std_logic;
signal btn_down : std_logic;
signal oe : std_logic;

signal timer_reg : unsigned(17 downto 0) := (others => '0');
signal timer_next : unsigned(17 downto 0);
signal upd_pulse : std_logic;

signal output_reg : std_logic_vector(12 downto 0) := (0 => '1', others => '0');
signal output_next : std_logic_vector(12 downto 0);


begin
  clk_mgnt : clk_wiz_0
  port map
  (
    clk_in1 => clk100,
    clk_out1 => clk5
  ); 

  matrix_cntrl : rgbm_cntr
  generic map
  (
    col_count => 64,
    rsize => 7
  )
  port map
  (
    clk_sys => clk5,
    reset => reset,
    oe_len => (0|1|2 => '1', others => '0'),
    led_x => x_addr,
    led_y => y_addr,
    pix_clk => open,
    pix_clk_g => output_next(10),
    stb => output_next(11),
    oe => oe
  );

  deb_reset : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => btnC,
    data_deb => reset
  );

  deb_btn_up : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => btnU,
    data_deb => btn_up
  );

  deb_btn_down : debounce
  generic map
  (
    num_cons_cycles => 100,
    cntr_size => 7
  )
  port map
  (
    clk => clk5,
    data_in => btnD,
    data_deb => btn_down
  );


  p_x <= unsigned(x_addr);
  p_y <= unsigned(y_addr);
  p_y_l <= p_y + 16;

  process (clk5)
  begin
    if rising_edge(clk5) then
      if reset = '1' then
        obj_reg_u <= "000"; -- blank screen
        obj_reg_l <= "000"; -- blank screen
        ball_xreg <= (others => '0');
        ball_yreg <= (others => '0');
        ball_dxreg <= to_unsigned(1, rsize);
        ball_dyreg <= to_unsigned(1, rsize);
        paddle_yreg <= to_unsigned(y_size/2 - paddle_height/2, rsize);
        timer_reg <= (others => '0');
        output_reg <= (0 => '1', others => '0');
      else
        obj_reg_u <= obj_next_u;
        obj_reg_l <= obj_next_l;
        ball_xreg <= ball_xnext;
        ball_yreg <= ball_ynext;
        ball_dxreg <= ball_dxnext;
        ball_dyreg <= ball_dynext;
        paddle_yreg <= paddle_ynext;
        timer_reg <= timer_next;
        output_reg <= output_next;
      end if;
    end if;
  end process;
  
  timer_next <= timer_reg + 1;
  upd_pulse <= '1' when timer_reg = 0 else '0';

  -- wall processing
  wall_active_u <= '1' when (wall_xl <= p_x) and (p_x <= wall_xr) else '0';
  wall_active_l <= '1' when (wall_xl <= p_x) and (p_x <= wall_xr) else '0';

  -- paddle processing
  paddle_yt <= paddle_yreg;
  paddle_yb <= paddle_yt + paddle_height - 1;
    
  -- paddle_active <= '1' when (p_x = to_unsigned(paddle_x, rsize))
  paddle_active_u <= '1' when (p_x = "0110111")
   and (((paddle_yt <= p_y) and (p_y <= paddle_yb))
   or ((paddle_yt <= p_y) and (p_y_l <= paddle_yb)))
   else '0';

  paddle_active_l <= '1' when (p_x = "0110111")
   and (((paddle_yt <= p_y_l) and (p_y_l <= paddle_yb))
   or ((paddle_yt <= p_y) and (p_y_l <= paddle_yb)))
   else '0';
  
  -- calculate new paddle position
  proc_paddle : process(paddle_yreg, paddle_yb, paddle_yt, upd_pulse, btn_up, btn_down)
  begin
    paddle_ynext <= paddle_yreg; -- default, no change
    if upd_pulse = '1' then
      -- move up only if enough space left between current bar position and top
      if (btn_up = '1') and (paddle_yt > paddle_d) then
        paddle_ynext <= paddle_yreg - paddle_d;
      --move down only if enough space left between current bar position and bottom
      elsif (btn_down = '1') and (paddle_yb < y_size - paddle_d - 1) then
        paddle_ynext <= paddle_yreg + paddle_d;
      end if;
    end if;
  end process proc_paddle;
        
  -- ball processing
  -- ball logic
  ball_xl <= ball_xreg;
  ball_xr <= ball_xl + ball_size - 1;
  ball_yt <= ball_yreg;
  ball_yb <= ball_yt + ball_size - 1;

  ball_active_u <= '1' when ((ball_xl <= p_x) and (p_x <= ball_xr))
    and (((ball_yt <= p_y) and (p_y <= ball_yb))
    or ((ball_yt <= p_y) and (p_y_l <= ball_yb)))
    else '0';

  ball_active_l <= '1' when ((ball_xl <= p_x) and (p_x <= ball_xr))
    and (((ball_yt <= p_y_l) and (p_y_l <= ball_yb))
    or ((ball_yt <= p_y) and (p_y_l <= ball_yb)))
    else '0';

  -- new ball position
  ball_xnext <= ball_xreg + ball_dxreg when upd_pulse = '1' else ball_xreg;
  ball_ynext <= ball_yreg + ball_dyreg when upd_pulse = '1' else ball_yreg;
  
  -- new ball velocity
  proc_ball : process(ball_dxreg, ball_dyreg, ball_xl, ball_xr, ball_yt, ball_yb, paddle_yt, paddle_yb)
  begin
    -- default values, in case no change occurs
    ball_dxnext <= ball_dxreg;
    ball_dynext <= ball_dyreg;
    
    if ball_yt < 1 then -- if ball reaches top display border
      ball_dynext <= ball_vp;
    elsif ball_yb > (y_size - 1) then
      ball_dynext <= ball_vn; -- if ball reaches botom display border
    elsif ball_xl < wall_xr then
      ball_dxnext <= ball_vp; -- ball reflected by wall
    elsif (paddle_x-1 < ball_xr) and (ball_xr <= paddle_x) then
      if (paddle_yt <= ball_yb) and (ball_yt <= paddle_yb) then
        ball_dxnext <= ball_vn; -- ball reflected by paddle
      end if;
    end if;
  end process proc_ball;
  
    -- determine display contents
  sel_proc_u : process (ball_active_u, wall_active_u, paddle_active_u)
  begin
    obj_next_u <= "000"; -- default, blank sceen
    if ball_active_u = '1' then
      obj_next_u <= ball_color;
    elsif wall_active_u = '1' then
      obj_next_u <= wall_color;
    elsif paddle_active_u = '1' then
      obj_next_u <= paddle_color;
    end if;
  end process sel_proc_u;
  
  sel_proc_l : process (ball_active_l, wall_active_l, paddle_active_l)
  begin
    obj_next_l <= "000"; -- default, blank sceen
    if ball_active_l = '1' then
      obj_next_l <= ball_color;
    elsif wall_active_l = '1' then
      obj_next_l <= wall_color;
    elsif paddle_active_l = '1' then
      obj_next_l <= paddle_color;
    end if;
  end process sel_proc_l;
    
  -- display update
  disp_yaddr <= y_addr(3 downto 0);
  disp_yaddr_minus_one <= std_logic_vector(unsigned(disp_yaddr) - 1);
  
  -- output data register update
  output_next(0) <= obj_reg_u(2);
  output_next(1) <= obj_reg_l(2);
  output_next(2) <= obj_reg_u(1);
  output_next(3) <= obj_reg_l(1);
  output_next(4) <= obj_reg_u(0);
  output_next(5) <= obj_reg_l(0);
  output_next(9 downto 6) <= disp_yaddr_minus_one(3 downto 0);
  -- bits 10 and 11 assigned in rgbm_cntr instantiation
  output_next(12) <= not oe;
  
  -- output dat -> ports
  -- upper half pixel data
  r0 <= output_reg(0);
  g0 <= output_reg(2);
  b0 <= output_reg(4);
  
  -- lower half pixel data
  r1 <= output_reg(1); 
  g1 <= output_reg(3);
  b1 <= output_reg(5);
  
  -- row addess
  a <= output_reg(6);
  b <= output_reg(7);
  c <= output_reg(8);
  d <= output_reg(9);
   
  -- data shift control
  clk_pix <= output_reg(10);
  stb <= output_reg(11);
  -- output enable 
  oe_n <= output_reg(12);

end impl;
