-- rgbm_cntrl.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
  );
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic;
    oe_len : in std_logic_vector(rsize-1 downto 0);
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end rgbm_cntr;

architecture impl of rgbm_cntr is

constant x_len : integer := col_count + stb_len + 1;
signal pclk_reg : std_logic := '0';
signal pclk_next : std_logic;
signal pix_pulse : std_logic;
signal xcntr_reg : unsigned(rsize-1 downto 0) := (others => '0');
signal xcntr_next : unsigned(rsize-1 downto 0);
signal x_end : std_logic;
signal ycntr_reg : unsigned(rsize-1 downto 0) := (others => '0');
signal ycntr_next : unsigned(rsize-1 downto 0);
signal stb_reg : std_logic := '0';
signal stb_next : std_logic;
signal oe_reg : std_logic := '0';
signal oe_next : std_logic;
signal clk_on : std_logic;

begin
  process(clk_sys)
  begin
    if rising_edge(clk_sys) then
      if reset = '1' then
        pclk_reg <= '0';
        xcntr_reg <= (others => '0');
        ycntr_reg <= (others => '0');
        stb_reg <= '0';
        oe_reg <= '0';
      else
        pclk_reg <= pclk_next;
        xcntr_reg <= xcntr_next;
        ycntr_reg <= ycntr_next;
        stb_reg <= stb_next;
        oe_reg <= oe_next;
      end if;
    end if;
  end process;
  
  pclk_next <= not pclk_reg; -- divide by 2
  pix_pulse <= '1' when pclk_reg = '1' else '0';
  x_end <= '1' when xcntr_reg = (x_len - 1) else '0';
  stb_next <= '1' when (xcntr_reg >= col_count) and
                       (xcntr_reg <= (col_count + stb_len - 1))
                  else '0';
  oe_next <= '1' when (xcntr_reg >= 1) and
                 (xcntr_reg <= (unsigned(oe_len) - 1))
                                  else '0';
  clk_on <= '1' when xcntr_reg <= (col_count - 1) else '0';
   
  proc_x : process (pix_pulse, xcntr_reg, x_end)
  begin
    xcntr_next <= xcntr_reg; -- default
    if pix_pulse = '1' then
      if x_end = '1' then
        xcntr_next <= (others => '0');
      else
        xcntr_next <= xcntr_reg + 1;
      end if;
    end if;
  end process;
  
  proc_y : process(x_end, ycntr_reg, pix_pulse)
  begin
    ycntr_next <= ycntr_reg; -- default
    if pix_pulse = '1' then
      if x_end = '1' then
        if ycntr_reg = row_count/rows_per_frame - 1 then
          ycntr_next <= (others => '0');
         else
           ycntr_next <= ycntr_reg + 1;
         end if;
      end if;
    end if;
  end process;
                          
  pix_clk <= pix_pulse;
  pix_clk_g <= pix_pulse and clk_on;
  led_x <= std_logic_vector(xcntr_reg);
  led_y <= std_logic_vector(ycntr_reg);
  stb <= stb_reg;
  oe <= oe_reg;
  
end impl;
